@@ -1,9 +1,15 @@
+from __future__ import annotations
+
 from datetime import timedelta
-from typing import List, Optional, Union
+from typing import TYPE_CHECKING
 
 import numpy as np
 
-from pandas._libs import lib, tslibs
+from pandas._libs import (
+    lib,
+    tslibs,
+)
+from pandas._libs.arrays import NDArrayBacked
 from pandas._libs.tslibs import (
     BaseOffset,
     NaT,
@@ -15,19 +21,26 @@ from pandas._libs.tslibs import (
     iNaT,
     to_offset,
 )
-from pandas._libs.tslibs.conversion import precision_from_unit
+from pandas._libs.tslibs.conversion import (
+    ensure_timedelta64ns,
+    precision_from_unit,
+)
 from pandas._libs.tslibs.fields import get_timedelta_field
 from pandas._libs.tslibs.timedeltas import (
     array_to_timedelta64,
     ints_to_pytimedelta,
     parse_timedelta_unit,
 )
+from pandas._typing import (
+    DtypeObj,
+    NpDtype,
+)
 from pandas.compat.numpy import function as nv
 
+from pandas.core.dtypes.cast import astype_td64_unit_conversion
 from pandas.core.dtypes.common import (
     DT64NS_DTYPE,
     TD64NS_DTYPE,
-    is_categorical_dtype,
     is_dtype_equal,
     is_float_dtype,
     is_integer_dtype,
@@ -35,21 +48,34 @@ from pandas.core.dtypes.common import (
     is_scalar,
     is_string_dtype,
     is_timedelta64_dtype,
-    is_timedelta64_ns_dtype,
     pandas_dtype,
 )
 from pandas.core.dtypes.dtypes import DatetimeTZDtype
-from pandas.core.dtypes.generic import ABCSeries, ABCTimedeltaIndex
+from pandas.core.dtypes.generic import (
+    ABCCategorical,
+    ABCMultiIndex,
+)
 from pandas.core.dtypes.missing import isna
 
 from pandas.core import nanops
 from pandas.core.algorithms import checked_add_with_arr
-from pandas.core.arrays import IntegerArray, datetimelike as dtl
+from pandas.core.arrays import (
+    ExtensionArray,
+    IntegerArray,
+    datetimelike as dtl,
+)
 from pandas.core.arrays._ranges import generate_regular_range
 import pandas.core.common as com
 from pandas.core.construction import extract_array
 from pandas.core.ops.common import unpack_zerodim_and_defer
 
+if TYPE_CHECKING:
+    from pandas import DataFrame
+    from pandas.core.arrays import (
+        DatetimeArray,
+        PeriodArray,
+    )
+
 
 def _field_accessor(name: str, alias: str, docstring: str):
     def f(self) -> np.ndarray:
@@ -71,8 +97,6 @@ class TimedeltaArray(dtl.TimelikeOps):
     """
     Pandas ExtensionArray for timedelta data.
 
-    .. versionadded:: 0.24.0
-
     .. warning::
 
        TimedeltaArray is currently experimental, and its API may change
@@ -108,12 +132,12 @@ class TimedeltaArray(dtl.TimelikeOps):
 
     __array_priority__ = 1000
     # define my properties & methods for delegation
-    _other_ops: List[str] = []
-    _bool_ops: List[str] = []
-    _object_ops = ["freq"]
-    _field_ops = ["days", "seconds", "microseconds", "nanoseconds"]
-    _datetimelike_ops = _field_ops + _object_ops + _bool_ops
-    _datetimelike_methods = [
+    _other_ops: list[str] = []
+    _bool_ops: list[str] = []
+    _object_ops: list[str] = ["freq"]
+    _field_ops: list[str] = ["days", "seconds", "microseconds", "nanoseconds"]
+    _datetimelike_ops: list[str] = _field_ops + _object_ops + _bool_ops
+    _datetimelike_methods: list[str] = [
         "to_pytimedelta",
         "total_seconds",
         "round",
@@ -124,11 +148,13 @@ class TimedeltaArray(dtl.TimelikeOps):
     # Note: ndim must be defined to ensure NaT.__richcmp__(TimedeltaArray)
     #  operates pointwise.
 
-    def _box_func(self, x) -> Union[Timedelta, NaTType]:
+    def _box_func(self, x) -> Timedelta | NaTType:
         return Timedelta(x, unit="ns")
 
     @property
-    def dtype(self) -> np.dtype:
+    # error: Return type "dtype" of "dtype" incompatible with return type
+    # "ExtensionDtype" in supertype "ExtensionArray"
+    def dtype(self) -> np.dtype:  # type: ignore[override]
         """
         The dtype for the TimedeltaArray.
 
@@ -147,8 +173,14 @@ class TimedeltaArray(dtl.TimelikeOps):
     # ----------------------------------------------------------------
     # Constructors
 
-    def __init__(self, values, dtype=TD64NS_DTYPE, freq=lib.no_default, copy=False):
-        values = extract_array(values)
+    _freq = None
+
+    def __init__(
+        self, values, dtype=TD64NS_DTYPE, freq=lib.no_default, copy: bool = False
+    ):
+        values = extract_array(values, extract_numpy=True)
+        if isinstance(values, IntegerArray):
+            values = values.to_numpy("int64", na_value=tslibs.iNaT)
 
         inferred_freq = getattr(values, "_freq", None)
         explicit_none = freq is None
@@ -163,12 +195,12 @@ class TimedeltaArray(dtl.TimelikeOps):
             elif freq and values.freq:
                 freq = to_offset(freq)
                 freq, _ = dtl.validate_inferred_freq(freq, values.freq, False)
-            values = values._data
+            values = values._ndarray
 
         if not isinstance(values, np.ndarray):
             msg = (
                 f"Unexpected type '{type(values).__name__}'. 'values' must be a "
-                "TimedeltaArray ndarray, or Series or Index containing one of those."
+                "TimedeltaArray, ndarray, or Series or Index containing one of those."
             )
             raise ValueError(msg)
         if values.ndim not in [1, 2]:
@@ -195,33 +227,29 @@ class TimedeltaArray(dtl.TimelikeOps):
         if freq:
             freq = to_offset(freq)
 
-        self._data = values
-        self._dtype = dtype
+        NDArrayBacked.__init__(self, values=values, dtype=dtype)
         self._freq = freq
 
         if inferred_freq is None and freq is not None:
             type(self)._validate_frequency(self, freq)
 
+    # error: Signature of "_simple_new" incompatible with supertype "NDArrayBacked"
     @classmethod
-    def _simple_new(
-        cls, values, freq: Optional[BaseOffset] = None, dtype=TD64NS_DTYPE
-    ) -> "TimedeltaArray":
+    def _simple_new(  # type: ignore[override]
+        cls, values: np.ndarray, freq: BaseOffset | None = None, dtype=TD64NS_DTYPE
+    ) -> TimedeltaArray:
         assert dtype == TD64NS_DTYPE, dtype
         assert isinstance(values, np.ndarray), type(values)
-        if values.dtype != TD64NS_DTYPE:
-            assert values.dtype == "i8"
-            values = values.view(TD64NS_DTYPE)
+        assert values.dtype == TD64NS_DTYPE
 
-        result = object.__new__(cls)
-        result._data = values
-        result._freq = to_offset(freq)
-        result._dtype = TD64NS_DTYPE
+        result = super()._simple_new(values=values, dtype=TD64NS_DTYPE)
+        result._freq = freq
         return result
 
     @classmethod
     def _from_sequence(
         cls, data, *, dtype=TD64NS_DTYPE, copy: bool = False
-    ) -> "TimedeltaArray":
+    ) -> TimedeltaArray:
         if dtype:
             _validate_td64_dtype(dtype)
 
@@ -238,7 +266,7 @@ class TimedeltaArray(dtl.TimelikeOps):
         copy: bool = False,
         freq=lib.no_default,
         unit=None,
-    ) -> "TimedeltaArray":
+    ) -> TimedeltaArray:
         if dtype:
             _validate_td64_dtype(dtype)
 
@@ -296,7 +324,7 @@ class TimedeltaArray(dtl.TimelikeOps):
         if not right_closed:
             index = index[:-1]
 
-        return cls._simple_new(index, freq=freq)
+        return cls._simple_new(index.view("m8[ns]"), freq=freq)
 
     # ----------------------------------------------------------------
     # DatetimeLike Interface
@@ -307,10 +335,10 @@ class TimedeltaArray(dtl.TimelikeOps):
         self._check_compatible_with(value, setitem=setitem)
         return np.timedelta64(value.value, "ns")
 
-    def _scalar_from_string(self, value):
+    def _scalar_from_string(self, value) -> Timedelta | NaTType:
         return Timedelta(value)
 
-    def _check_compatible_with(self, other, setitem: bool = False):
+    def _check_compatible_with(self, other, setitem: bool = False) -> None:
         # we don't have anything to validate.
         pass
 
@@ -324,22 +352,9 @@ class TimedeltaArray(dtl.TimelikeOps):
         # DatetimeLikeArrayMixin super call handles other cases
         dtype = pandas_dtype(dtype)
 
-        if is_timedelta64_dtype(dtype) and not is_timedelta64_ns_dtype(dtype):
-            # by pandas convention, converting to non-nano timedelta64
-            #  returns an int64-dtyped array with ints representing multiples
-            #  of the desired timedelta unit.  This is essentially division
-            if self._hasnans:
-                # avoid double-copying
-                result = self._data.astype(dtype, copy=False)
-                return self._maybe_mask_results(
-                    result, fill_value=None, convert="float64"
-                )
-            result = self._data.astype(dtype, copy=copy)
-            return result.astype("i8")
-        elif is_timedelta64_ns_dtype(dtype):
-            if copy:
-                return self.copy()
-            return self
+        if dtype.kind == "m":
+            return astype_td64_unit_conversion(self._ndarray, dtype, copy=copy)
+
         return dtl.DatetimeLikeArrayMixin.astype(self, dtype, copy=copy)
 
     def __iter__(self):
@@ -351,7 +366,7 @@ class TimedeltaArray(dtl.TimelikeOps):
             data = self.asi8
             length = len(self)
             chunksize = 10000
-            chunks = int(length / chunksize) + 1
+            chunks = (length // chunksize) + 1
             for i in range(chunks):
                 start_i = i * chunksize
                 end_i = min((i + 1) * chunksize, length)
@@ -364,8 +379,8 @@ class TimedeltaArray(dtl.TimelikeOps):
     def sum(
         self,
         *,
-        axis=None,
-        dtype=None,
+        axis: int | None = None,
+        dtype: NpDtype | None = None,
         out=None,
         keepdims: bool = False,
         initial=None,
@@ -384,8 +399,8 @@ class TimedeltaArray(dtl.TimelikeOps):
     def std(
         self,
         *,
-        axis=None,
-        dtype=None,
+        axis: int | None = None,
+        dtype: NpDtype | None = None,
         out=None,
         ddof: int = 1,
         keepdims: bool = False,
@@ -403,16 +418,19 @@ class TimedeltaArray(dtl.TimelikeOps):
     # ----------------------------------------------------------------
     # Rendering Methods
 
-    def _formatter(self, boxed=False):
+    def _formatter(self, boxed: bool = False):
         from pandas.io.formats.format import get_format_timedelta64
 
         return get_format_timedelta64(self, box=True)
 
-    def _format_native_types(self, na_rep="NaT", date_format=None, **kwargs):
+    @dtl.ravel_compat
+    def _format_native_types(
+        self, na_rep="NaT", date_format=None, **kwargs
+    ) -> np.ndarray:
         from pandas.io.formats.format import get_format_timedelta64
 
-        formatter = get_format_timedelta64(self._data, na_rep)
-        return np.array([formatter(x) for x in self._data.ravel()]).reshape(self.shape)
+        formatter = get_format_timedelta64(self._ndarray, na_rep)
+        return np.array([formatter(x) for x in self._ndarray])
 
     # ----------------------------------------------------------------
     # Arithmetic Methods
@@ -423,12 +441,12 @@ class TimedeltaArray(dtl.TimelikeOps):
             f"cannot add the type {type(other).__name__} to a {type(self).__name__}"
         )
 
-    def _add_period(self, other: Period):
+    def _add_period(self, other: Period) -> PeriodArray:
         """
         Add a Period object.
         """
         # We will wrap in a PeriodArray and defer to the reversed operation
-        from .period import PeriodArray
+        from pandas.core.arrays.period import PeriodArray
 
         i8vals = np.broadcast_to(other.ordinal, self.shape)
         oth = PeriodArray(i8vals, freq=other.freq)
@@ -447,7 +465,7 @@ class TimedeltaArray(dtl.TimelikeOps):
         # defer to implementation in DatetimeArray
         return other + self
 
-    def _add_datetimelike_scalar(self, other):
+    def _add_datetimelike_scalar(self, other) -> DatetimeArray:
         # adding a timedeltaindex to a datetimelike
         from pandas.core.arrays import DatetimeArray
 
@@ -478,10 +496,10 @@ class TimedeltaArray(dtl.TimelikeOps):
             ) from err
 
     @unpack_zerodim_and_defer("__mul__")
-    def __mul__(self, other) -> "TimedeltaArray":
+    def __mul__(self, other) -> TimedeltaArray:
         if is_scalar(other):
             # numpy will accept float and int, raise TypeError for others
-            result = self._data * other
+            result = self._ndarray * other
             freq = None
             if self.freq is not None and not isna(other):
                 freq = self.freq * other
@@ -504,7 +522,7 @@ class TimedeltaArray(dtl.TimelikeOps):
             return type(self)(result)
 
         # numpy will accept float or int dtype, raise TypeError for others
-        result = self._data * other
+        result = self._ndarray * other
         return type(self)(result)
 
     __rmul__ = __mul__
@@ -522,11 +540,11 @@ class TimedeltaArray(dtl.TimelikeOps):
                 return result
 
             # otherwise, dispatch to Timedelta implementation
-            return self._data / other
+            return self._ndarray / other
 
         elif lib.is_scalar(other):
             # assume it is numeric
-            result = self._data / other
+            result = self._ndarray / other
             freq = None
             if self.freq is not None:
                 # Tick division is not implemented, so operate on Timedelta
@@ -542,7 +560,7 @@ class TimedeltaArray(dtl.TimelikeOps):
 
         elif is_timedelta64_dtype(other.dtype):
             # let numpy handle it
-            return self._data / other
+            return self._ndarray / other
 
         elif is_object_dtype(other.dtype):
             # We operate on raveled arrays to avoid problems in inference
@@ -564,7 +582,7 @@ class TimedeltaArray(dtl.TimelikeOps):
             return result
 
         else:
-            result = self._data / other
+            result = self._ndarray / other
             return type(self)(result)
 
     @unpack_zerodim_and_defer("__rtruediv__")
@@ -579,7 +597,7 @@ class TimedeltaArray(dtl.TimelikeOps):
                 return result
 
             # otherwise, dispatch to Timedelta implementation
-            return other / self._data
+            return other / self._ndarray
 
         elif lib.is_scalar(other):
             raise TypeError(
@@ -595,7 +613,7 @@ class TimedeltaArray(dtl.TimelikeOps):
 
         elif is_timedelta64_dtype(other.dtype):
             # let numpy handle it
-            return other / self._data
+            return other / self._ndarray
 
         elif is_object_dtype(other.dtype):
             # Note: unlike in __truediv__, we do not _need_ to do type
@@ -622,7 +640,7 @@ class TimedeltaArray(dtl.TimelikeOps):
                     return result
 
                 # dispatch to Timedelta implementation
-                result = other.__rfloordiv__(self._data)
+                result = other.__rfloordiv__(self._ndarray)
                 return result
 
             # at this point we should only have numeric scalars; anything
@@ -658,7 +676,11 @@ class TimedeltaArray(dtl.TimelikeOps):
             return result
 
         elif is_object_dtype(other.dtype):
-            result = [self[n] // other[n] for n in range(len(self))]
+            # error: Incompatible types in assignment (expression has type
+            # "List[Any]", variable has type "ndarray")
+            result = [  # type: ignore[assignment]
+                self[n] // other[n] for n in range(len(self))
+            ]
             result = np.array(result)
             if lib.infer_dtype(result, skipna=False) == "timedelta":
                 result, _ = sequence_to_td64ns(result)
@@ -666,7 +688,7 @@ class TimedeltaArray(dtl.TimelikeOps):
             return result
 
         elif is_integer_dtype(other.dtype) or is_float_dtype(other.dtype):
-            result = self._data // other
+            result = self._ndarray // other
             return type(self)(result)
 
         else:
@@ -686,7 +708,7 @@ class TimedeltaArray(dtl.TimelikeOps):
                     return result
 
                 # dispatch to Timedelta implementation
-                result = other.__floordiv__(self._data)
+                result = other.__floordiv__(self._ndarray)
                 return result
 
             raise TypeError(
@@ -712,7 +734,11 @@ class TimedeltaArray(dtl.TimelikeOps):
             return result
 
         elif is_object_dtype(other.dtype):
-            result = [other[n] // self[n] for n in range(len(self))]
+            # error: Incompatible types in assignment (expression has type
+            # "List[Any]", variable has type "ndarray")
+            result = [  # type: ignore[assignment]
+                other[n] // self[n] for n in range(len(self))
+            ]
             result = np.array(result)
             return result
 
@@ -754,17 +780,17 @@ class TimedeltaArray(dtl.TimelikeOps):
         res2 = other - res1 * self
         return res1, res2
 
-    def __neg__(self) -> "TimedeltaArray":
+    def __neg__(self) -> TimedeltaArray:
         if self.freq is not None:
-            return type(self)(-self._data, freq=-self.freq)
-        return type(self)(-self._data)
+            return type(self)(-self._ndarray, freq=-self.freq)
+        return type(self)(-self._ndarray)
 
-    def __pos__(self) -> "TimedeltaArray":
-        return type(self)(self._data, freq=self.freq)
+    def __pos__(self) -> TimedeltaArray:
+        return type(self)(self._ndarray, freq=self.freq)
 
-    def __abs__(self) -> "TimedeltaArray":
+    def __abs__(self) -> TimedeltaArray:
         # Note: freq is not preserved
-        return type(self)(np.abs(self._data))
+        return type(self)(np.abs(self._ndarray))
 
     # ----------------------------------------------------------------
     # Conversion Methods - Vectorized analogues of Timedelta methods
@@ -833,7 +859,7 @@ class TimedeltaArray(dtl.TimelikeOps):
 
         Returns
         -------
-        datetimes : ndarray
+        timedeltas : ndarray[object]
         """
         return tslibs.ints_to_pytimedelta(self.asi8)
 
@@ -855,14 +881,14 @@ class TimedeltaArray(dtl.TimelikeOps):
     )
 
     @property
-    def components(self):
+    def components(self) -> DataFrame:
         """
         Return a dataframe of the components (days, hours, minutes,
         seconds, milliseconds, microseconds, nanoseconds) of the Timedeltas.
 
         Returns
         -------
-        a DataFrame
+        DataFrame
         """
         from pandas import DataFrame
 
@@ -898,7 +924,9 @@ class TimedeltaArray(dtl.TimelikeOps):
 # Constructor Helpers
 
 
-def sequence_to_td64ns(data, copy=False, unit=None, errors="raise"):
+def sequence_to_td64ns(
+    data, copy: bool = False, unit=None, errors="raise"
+) -> tuple[np.ndarray, Tick | None]:
     """
     Parameters
     ----------
@@ -940,17 +968,23 @@ def sequence_to_td64ns(data, copy=False, unit=None, errors="raise"):
             # i.e. generator
             data = list(data)
         data = np.array(data, copy=False)
-    elif isinstance(data, ABCSeries):
-        data = data._values
-    elif isinstance(data, (ABCTimedeltaIndex, TimedeltaArray)):
-        inferred_freq = data.freq
-        data = data._data
-    elif isinstance(data, IntegerArray):
-        data = data.to_numpy("int64", na_value=tslibs.iNaT)
-    elif is_categorical_dtype(data.dtype):
+    elif isinstance(data, ABCMultiIndex):
+        raise TypeError("Cannot create a DatetimeArray from a MultiIndex.")
+    else:
+        data = extract_array(data, extract_numpy=True)
+
+    if isinstance(data, IntegerArray):
+        data = data.to_numpy("int64", na_value=iNaT)
+    elif not isinstance(data, (np.ndarray, ExtensionArray)):
+        # GH#24539 e.g. xarray, dask object
+        data = np.asarray(data)
+    elif isinstance(data, ABCCategorical):
         data = data.categories.take(data.codes, fill_value=NaT)._values
         copy = False
 
+    if isinstance(data, TimedeltaArray):
+        inferred_freq = data.freq
+
     # Convert whatever we have into timedelta64[ns] dtype
     if is_object_dtype(data.dtype) or is_string_dtype(data.dtype):
         # no need to make a copy, need to convert if string-dtyped
@@ -978,8 +1012,7 @@ def sequence_to_td64ns(data, copy=False, unit=None, errors="raise"):
     elif is_timedelta64_dtype(data.dtype):
         if data.dtype != TD64NS_DTYPE:
             # non-nano unit
-            # TODO: watch out for overflows
-            data = data.astype(TD64NS_DTYPE)
+            data = ensure_timedelta64ns(data)
             copy = False
 
     else:
@@ -1021,8 +1054,8 @@ def ints_to_td64ns(data, unit="ns"):
         dtype_str = f"timedelta64[{unit}]"
         data = data.view(dtype_str)
 
-        # TODO: watch out for overflows when converting from lower-resolution
-        data = data.astype("timedelta64[ns]")
+        data = ensure_timedelta64ns(data)
+
         # the astype conversion makes a copy, so we can avoid re-copying later
         copy_made = True
 
@@ -1068,7 +1101,7 @@ def objects_to_td64ns(data, unit=None, errors="raise"):
     return result.view("timedelta64[ns]")
 
 
-def _validate_td64_dtype(dtype):
+def _validate_td64_dtype(dtype) -> DtypeObj:
     dtype = pandas_dtype(dtype)
     if is_dtype_equal(dtype, np.dtype("timedelta64")):
         # no precision disallowed GH#24806
